@@ -44,7 +44,8 @@ fuzz_words_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), os.pat
 fuzz_types = [os.path.join("fuzzdb","attack")]#os.listdir(os.path.join(root, "fuzzdb/attack"))#["general", "injections", "vulns", "webservices", "stress", "others"]
 supoorted_fuzz_types = [_ for _ in os.listdir(fuzz_words_dir) if "README" not in _]
 ANOMOLY_THRESHOLD = 1.07142
-RESERVED_PARAMETERS = ["$RESPONSE_ERROR", "$CV_EVENT_ID"]
+RESERVED_PARAMETERS = ["$RESPONSE_ERROR", "$CV_EVENT_ID", "$headers"]
+test_data_generation_summary = []
 
 def update_nested_dict(key, value, dictionary):
     for k, v in dictionary.items():
@@ -409,11 +410,11 @@ class CloudvectorDAST(object):
                 for each in v:
                     if type(each) is dict:
                         for a, b in keypaths(each):
-                            list(update_nested_dict(a[-1], "$" + a[-1] + "$", each))
+                            list(update_nested_dict(a[-1], "$_.get('" + a[-1] + "')$", each))
                     else:
-                        list(update_nested_dict(k[-1], "$" + k[-1] + "$", self.params_details_from_spec))
+                        list(update_nested_dict(k[-1], "$_.get('" + k[-1] + "')$", self.params_details_from_spec))
             elif type(v) is dict:
-                list(update_nested_dict(a[-1], "$" + a[-1] + "$", v))
+                list(update_nested_dict(a[-1], "$_.get('" + a[-1] + "')$", v))
 
         params_for_fixtures = []
         for api, info in self.params_captured_in_traffic.items():
@@ -434,6 +435,13 @@ class CloudvectorDAST(object):
                                      AUTH_API_HEADERS=auth_api_headers, AUTH_API_PAYLOAD=json.dumps(auth_api_payload),
                                      AUTH_RESP_KEY=auth_api_response_key, AUTH_INPUTS=auth_inputs,
                                      TOKEN_PREFIX=auth_prefix))
+
+        with open(os.path.join(templates_dir, 'cvutils.j2')) as file_:
+            template = Template(file_.read())
+
+        with open("tests/cvutils.py", 'w+') as fh:
+            fh.write(template.render(fuzz_supported=supoorted_fuzz_types))
+
         print("\n\t\t......done creating pytest fixtures (conftest.py)")
 
     def create_fuzzfixtures(self):
@@ -609,13 +617,12 @@ class CloudvectorDAST(object):
         print("\n\ncreating Pytest test methods for fuzzing....\n")
         host_url, apis_to_be_tested = self._process_cv_events(cv_events)
 
-        if self.ceobj:
-            statefull = 1
-        else:
+        statefull = 1
+        if not self.ce_obj:
             statefull = 0
-
-        if os.environ.get("CVIAST_STATEFUL_FUZZ","3") in ["0", "1"]:
-            statefull = int(os.environ.get("CVIAST_STATEFUL_FUZZ"))
+        else:
+            if os.environ.get("CVIAST_STATEFUL_FUZZ","3") in ["0", "1"]:
+                statefull = int(os.environ.get("CVIAST_STATEFUL_FUZZ"))
 
         if os.path.exists("tests/data"):
             shutil.rmtree("tests/data")
@@ -678,17 +685,18 @@ class CloudvectorDAST(object):
                 continue
             if int(str(event["attributes"]["http_rsp_status_code"]).split(" ")[0]) == 0:
                 continue
-            if apis_to_check:
-                iflag = False
-                for _ in apis_to_check:
-                    if str(_).lower() in str(event["attributes"]["http_path"]).lower():
-                        iflag = True
-                        params_to_add = apis_to_check[_]
-                    if iflag:
-                        break
+            # if apis_to_check:
+            #     iflag = False
+            #     for _ in apis_to_check:
+            #         if str(_).lower() in str(event["attributes"]["http_path"]).lower():
+            #             iflag = True
+            #             params_to_add = apis_to_check[_]
+            #         if iflag:
+            #             break
+            #
+            #     if not iflag:
+            #         continue
 
-                if not iflag:
-                    continue
             request = {"url": str(event["attributes"]["http_path"]),
                        "method": str(event["attributes"]["http_method"]).lower()}
             header = {}
@@ -738,6 +746,24 @@ class CloudvectorDAST(object):
                 request["rsp_error"] = event["attributes"]["event_json"].get("http-rsp-body-error")
             url = event["attributes"]["http_path"]
 
+            api_queried = event.get("api_queried")
+            if api_queried:
+                if "{" in api_queried:
+                    api_queried = api_queried.split("/")
+                    api_queried.remove("")
+                    event_url = url.split("/")
+                    event_url.remove("")
+                    if api_queried[0] in event_url:
+                        start_index = event_url.index(api_queried[0])
+                        for p in api_queried[1:]:
+                            if p.startswith("{"):
+                                req_params_found[p.replace("{","").replace("}","")] = event_url[start_index+1]
+                            start_index+=1
+                        url = "/".join(event_url[:event_url.index(api_queried[0])])+event.get("api_queried")
+                        if not url.startswith("/"):
+                            url = "/" + url
+                        request["http-req-url"] = url
+                        url = url.replace("{","").replace("}","")
             if str(url) not in self.params_captured_in_traffic:
                 self.params_captured_in_traffic[url] = {}
             if str(request["rsp_code"]) not in self.params_captured_in_traffic[url]:
@@ -747,6 +773,8 @@ class CloudvectorDAST(object):
             params_found = {}
             for param, value in req_params_found.items():
                 params_found[param] = value
+            params_found["header"] = header
+            params_found["header"]["Authorization"] = ""
             if params_found:
                 if request["rsp_code"] not in [200, 201]:
                     params_found["$RESPONSE_ERROR"] = request.get("rsp_error")
@@ -775,11 +803,13 @@ class CloudvectorDAST(object):
 
     @staticmethod
     def generate_fuzz_report(output_report):
-        if not os.path.exists(os.path.join(os.getcwd(), 'tests/data/attack_info.json')):
-            print("\n\n\tThe flag '--generate-report' is a beta version only applicable for FUZZ tests at the moment!\n")
-            return
-        with open(os.path.join(os.getcwd(), 'tests/data/attack_info.json')) as file_:
-            attack_info = json.load(file_).get("info")
+        # if not os.path.exists(os.path.join(os.getcwd(), 'tests/data/attack_info.json')):
+        #     print("\n\n\tThe flag '--generate-report' is a beta version only applicable for FUZZ tests at the moment!\n")
+        #     return
+        attack_info = {}
+        if os.path.exists(os.path.join(os.getcwd(), 'tests/data/attack_info.json')):
+            with open(os.path.join(os.getcwd(), 'tests/data/attack_info.json')) as file_:
+                attack_info = json.load(file_).get("info")
 
         with open("report.json") as fobj:
             report = json.load(fobj)
@@ -946,10 +976,11 @@ class CloudvectorDAST(object):
 
         fuzz_data = {}
         total_apis = 0
-
+        apis_tested = []
         for api, info in consolidated_report.items():
             for method, fuzz_info in info.items():
                 total_apis += 1
+                apis_tested.append(api.split("[")[0])
                 for type, info in fuzz_info.items():
                     total_tests = 0
                     success_tests = 0
@@ -992,6 +1023,7 @@ class CloudvectorDAST(object):
                                                "success_apis": v["apis_tested"] - v["apis_failed"],
                                                "failed_apis": v["apis_failed"], "app":"",
                                                "total_tests": v["total_tests"], "success_tests": v["success_tests"],
+                                               "skipped_tests": int(v["total_tests"]) - int(v["success_tests"]) - int(v["failed_tests"]),
                                                "failed_tests": v["failed_tests"],"resp_length":v["resp_length"],
                                                "resp_duration":v["resp_duration"]})
             else:
@@ -1029,7 +1061,7 @@ class CloudvectorDAST(object):
         with open(output_report, "wb+") as file_:
             file_.write(template.render(api_data=api_data, fuzz_data=fuzz_data_processed, total_api_calls=total_api_calls,
                                         total_vulnerabilities=vulnerabilities_found, spaarc_findings=spaarc_findings,
-                                        test_time=datetime.datetime.fromtimestamp(time.time()).isoformat(), total_apis_scanned=total_apis,
+                                        test_time=datetime.datetime.fromtimestamp(time.time()).isoformat(), total_apis_scanned=len(list(set(apis_tested))),
                                         fuzz_types=fuzz_types, vulns_stats=vulns_stats, fuzz_metadata=fuzz_metainfo,
                                         detailed_report=os.path.join(os.getcwd(),"CVIAST-detailed_report.html"),
                                         test_lapse_time=str(round(int(report["report"]["summary"]["duration"])/60))+" min(s)").encode("utf-8","ignore"))
@@ -1037,12 +1069,19 @@ class CloudvectorDAST(object):
 
 
     @staticmethod
-    def create_test_data_sets(api, method):
+    def create_test_data_sets(api, method, status_code="200"):
         with open("tests/params_captured.json") as fobj:
             f = json.load(fobj)
         params_collection = []
-        for _ in f.get(api, {}).get("200", {}).get(method, {}):
+        for _ in f.get(api, {}).get(status_code, {}).get(method, {}):
             params_info = {}
+            header = _.get("header")
+            rsp_error = _.get("$RESPONSE_ERROR")
+            if rsp_error:
+                del _["$RESPONSE_ERROR"]
+            if header:
+                del _["header"]
+
             for k, v in keypaths(_):
                 if type(v) is list:
                     for e in v:
@@ -1053,18 +1092,24 @@ class CloudvectorDAST(object):
                             params_info[k[-1]] = v
                 else:
                     params_info[k[-1]] = v
+            params_info["header"] = header
+            params_info["$RESPONSE_ERROR"] = rsp_error
             params_collection.append(params_info)
 
+        if not status_code.startswith("2"):
+            return params_collection
+
         input_params = {}
         processed_params = []
         selected_events_data = []
+
         for collection in params_collection:
             for k, v in collection.items():
-                if k.startswith("$"):
+                if k.startswith("$") or k=="header":
                     continue
                 if k not in input_params:
                     input_params[k] = {}
-                    continue
+                    #continue
                 if type(v) is list:
                     if len(v) > len(input_params.get(k, {}).get(k, [])):
                         processed_params.append(k)
@@ -1074,6 +1119,7 @@ class CloudvectorDAST(object):
                                 len(v)) + " with " \
                                           "values " + str(v)
                             input_params[k] = new_collection
+                            test_data_generation_summary.append(str(api)+"-->"+str(method)+"--> 200 -->"+str(new_collection["$comment"]))
                 elif type(v) is bool:
                     continue
                 elif type(v) is int:
@@ -1083,18 +1129,26 @@ class CloudvectorDAST(object):
                             new_collection = deepcopy(collection)
                             new_collection["$comment"] = "Maximum value for " + str(k) + " is " + str(v)
                             input_params[k] = new_collection
+                            test_data_generation_summary.append(str(api)+"-->"+str(method)+"--> 200 -->"+str(new_collection["$comment"]))
                 elif len(str(v)) > len(str(input_params.get(k, {}).get(k, ""))):
+                    # try:
+                    #     datetime.datetime.strptime(str(v), '%Y-%m-%dT%H:%M:%SZ')
+                    #     continue
+                    # except:
+                    #     pass
                     processed_params.append(k)
                     if input_params:
                         new_collection = deepcopy(collection)
                         new_collection["$comment"] = "Maximum value for " + str(k) + " is " + str(
                             v) + " with length " + str(len(str(v)))
                         input_params[k] = new_collection
+                        test_data_generation_summary.append(
+                            str(api) + "-->" + str(method) + "--> 200 -->" + str(new_collection["$comment"]))
         selected_events_data += list(input_params.values())
 
         for collection in params_collection:
             for k, v in collection.items():
-                if k.startswith("$"):
+                if k.startswith("$") or "date" in str(k).lower() or "time" in str(k).lower():
                     continue
                 if k not in input_params:
                     input_params[k] = {}
@@ -1114,15 +1168,23 @@ class CloudvectorDAST(object):
                                 len(v)) + " with " \
                                           "values " + str(v)
                             input_params[k] = new_collection
+                            test_data_generation_summary.append(str(api)+"-->"+str(method)+"--> 200 -->"+str(new_collection["$comment"]))
                 elif type(v) is bool:
                     continue
                 elif type(v) is int:
-                    if v < int(input_params.get(k, {}).get(k, 0)):
+                    to_check = input_params.get(k, {}).get(k, 0)
+                    try:
+                        to_check = int(to_check)
+                    except ValueError:
+                        to_check = str(to_check)
+                        v = str(v)
+                    if v < to_check:
                         processed_params.append(k)
                         if input_params:
                             new_collection = deepcopy(collection)
                             new_collection["$comment"] = "Minimum value for " + str(k) + " is " + str(v)
                             input_params[k] = new_collection
+                            test_data_generation_summary.append(str(api)+"-->"+str(method)+"--> 200 -->"+str(new_collection["$comment"]))
                 elif len(str(v)) < len(str(input_params.get(k, {}).get(k, ""))):
                     processed_params.append(k)
                     if input_params:
@@ -1130,6 +1192,8 @@ class CloudvectorDAST(object):
                         new_collection["$comment"] = "Minimum value for " + str(k) + " is " + str(
                             v) + " with length " + str(len(str(v)))
                         input_params[k] = new_collection
+                        test_data_generation_summary.append(
+                            str(api) + "-->" + str(method) + "--> 200 -->" + str(new_collection["$comment"]))
         selected_events_data += list(input_params.values())
         return selected_events_data
 
@@ -1144,21 +1208,70 @@ class CloudvectorDAST(object):
             print("Error: You need to have used a valid SPEC to generate the tests to know the parameter details, Please regenerate tests!!!")
             return
         input_params = {}
+        fourhundred_data_set = {}
+        fivehundred_data_set = {}
+        data_generation_summary = {}
         for api, info in f1.items():
+            api = api.replace("{","").replace("}","")
             if api not in input_params:
                 input_params[api] = {"200": {}, "400": {}, "401": {}}
             for method, method_info in info.items():
                 data_set = CloudvectorDAST.create_test_data_sets(api, method)
+                if api not in fourhundred_data_set:
+                    fourhundred_data_set[api] = {"400":{}}
+                if method not in fourhundred_data_set[api]["400"]:
+                    fourhundred_data_set[api]["400"][method]=CloudvectorDAST.create_test_data_sets(api, method, "400")
+                if api not in fivehundred_data_set:
+                    fivehundred_data_set[api] = {"500":{}}
+                if method not in fivehundred_data_set[api]["500"]:
+                    fivehundred_data_set[api]["500"][method]=CloudvectorDAST.create_test_data_sets(api, method, "500")
                 if not data_set:
                     continue
                 random_data_set = random.choice(data_set)
+                start_datetime_key = ""
+                end_datetime_key = ""
+                for k, v in random_data_set.items():
+                    if "starttime" in str(k).lower() or "startdate" in str(k).lower():
+                        start_datetime_key = k
+                    if "endtime" in str(k).lower() or "enddate" in str(k).lower():
+                        end_datetime_key = k
+                header = random_data_set.get("header")
+                if header:
+                    del random_data_set["header"]
+                #     random_data_set.update(header)
                 for k, v in random_data_set.items():
                     if k.startswith("$"):
                         continue
                     if k in method_info.get("required"):
+                        error_data_block = deepcopy(random_data_set)
+                        if k == start_datetime_key and end_datetime_key:
+                            error_data_block[k] = error_data_block[end_datetime_key]
+                            error_data_block[end_datetime_key] = v
+                            error_data_block["$comment"] = "This payload will swap the value of "+str(start_datetime_key)+" to "+str(end_datetime_key)
+                            if header:
+                                error_data_block.update(header)
+                                error_data_block["Authorization"] = ""
+                                error_data_block["headers"] = list(header.keys())
+                            if method not in input_params[api]["400"]:
+                                input_params[api]["400"][method] = [error_data_block]
+                            else:
+                                input_params[api]["400"][method].append(error_data_block)
+                        # elif k == end_datetime_key and start_datetime_key:
+                        #     error_data_block[k] = error_data_block.get(start_datetime_key)
+                        #     error_data_block[start_datetime_key] = v
+                        #     error_data_block["$comment"] = "This payload will swap the value of " + str(
+                        #         end_datetime_key) + " to " + str(start_datetime_key)
                         error_data_block = deepcopy(random_data_set)
                         error_data_block[k] = "<delete>"
                         error_data_block["$comment"] = "This payload will miss the mandatory parameter " + str(k)
+                        test_data_generation_summary.append(
+                            str(api) + "-->" + str(method) + "--> 400 -->" + str(error_data_block["$comment"]))
+                        #header = error_data_block.get("header")
+                        if header:
+                            error_data_block.update(header)
+                            error_data_block["Authorization"] = ""
+                            error_data_block["headers"] = list(header.keys())
+                            error_data_block = collections.OrderedDict(reversed(sorted(error_data_block.items())))
                         if method not in input_params[api]["400"]:
                             input_params[api]["400"][method] = [error_data_block]
                         else:
@@ -1167,16 +1280,99 @@ class CloudvectorDAST(object):
                         success_data_block = deepcopy(random_data_set)
                         success_data_block[k] = "<delete>"
                         success_data_block["$comment"] = "This payload will miss the optional parameter " + str(k)
+                        test_data_generation_summary.append(
+                            str(api) + "-->" + str(method) + "--> 200 -->" + str(success_data_block["$comment"]))
+                        #header = success_data_block.get("header")
+                        if header:
+                            success_data_block.update(header)
+                            success_data_block["Authorization"] = ""
+                            success_data_block["$headers"] = list(header.keys())
+                            success_data_block = collections.OrderedDict(reversed(sorted(success_data_block.items())))
                         if method not in input_params[api]["200"]:
                             input_params[api]["200"][method] = [success_data_block]
                         else:
                             input_params[api]["200"][method].append(success_data_block)
 
+                for _ in range(len(data_set)):
+                    header = data_set[_].get("header")
+                    if header:
+                        del data_set[_]["header"]
+                        data_set[_].update(header)
+                        data_set[_]["Authorization"] = ""
+                        data_set[_]["$headers"] = list(header.keys())
+                        data_set[_] = collections.OrderedDict(reversed(sorted(data_set[_].items())))
                 if method not in input_params[api]["200"]:
                     input_params[api]["200"][method] = data_set
                 else:
                     input_params[api]["200"][method] += data_set
 
+                if method not in input_params[api]["401"]:
+                    rand_choice = random.choice(data_set)
+                    if rand_choice:
+                        rand_choice["Authorization"] = ""
+                        input_params[api]["401"][method] = [rand_choice]
+
+                for api, info in fourhundred_data_set.items():
+                    for status, method_info in info.items():
+                        for method, info in method_info.items():
+                            rsp_error_codes_found = []
+                            for each in info:
+                                if each.get("$RESPONSE_ERROR") and "code" in each.get("$RESPONSE_ERROR"):
+                                    error_code = each.get("$RESPONSE_ERROR",{}).get("code")
+                                else:
+                                    continue
+                                if error_code not in rsp_error_codes_found:
+                                    rsp_error_codes_found.append(error_code)
+                                    each["$comment"] = "picked for the error code: "+str(each["$RESPONSE_ERROR"]["code"])
+                                    test_data_generation_summary.append(
+                                        str(api) + "-->" + str(method) + "--> 400 -->" + str(each["$comment"]))
+                                    if method not in input_params[api]["400"]:
+                                        input_params[api]["400"][method] = each
+
+                for api, info in fivehundred_data_set.items():
+                    for status, method_info in info.items():
+                        for method, info in method_info.items():
+                            rsp_error_codes_found = []
+                            for each in info:
+                                if each.get("$RESPONSE_ERROR") and "code" in each.get("$RESPONSE_ERROR"):
+                                    error_code = each.get("$RESPONSE_ERROR",{}).get("code")
+                                else:
+                                    continue
+                                if error_code not in rsp_error_codes_found:
+                                    rsp_error_codes_found.append(error_code)
+                                    each["$comment"] = "picked for the error code: "+str(each["$RESPONSE_ERROR"]["code"])
+                                    test_data_generation_summary.append(
+                                        str(api) + "-->" + str(method) + "--> 500 -->" + str(each["$comment"]))
+                                    if "500" not in input_params[api]:
+                                        input_params[api]["500"] = {}
+                                    if method not in input_params[api]["500"]:
+                                        input_params[api]["500"][method] = each
+
+        summary = {}
+        for k in test_data_generation_summary:
+            api, method, status_code, comment = k.split("-->")
+            print(api, method, status_code, comment,"\n\n\n")
+            if api not in summary:
+                summary[api] = {}
+            if method not in summary[api]:
+                summary[api][method] = {}
+            if status_code not in summary[api][method]:
+                summary[api][method][status_code] = []
+            summary[api][method][status_code].append(comment)
+
+        total_tests = 0
+        for api, info in summary.items():
+            for method, minfo in info.items():
+                print(api+" --> "+method)
+                for code, dset in minfo.items():
+                    dset = list(set(dset))
+                    print("\t\t\t\tResponse Code to be triggered: "+str(code))
+                    print("\t\t\t\t# of data sets(test cases): "+str(len(dset)))
+                    print("\t\t\t\tReason(s) for the Test data: " + str(dset))
+                    total_tests+=len(dset)
+            print("-------"*15)
+
+
         if os.path.exists("tests"):
             with open("tests/input_params_auto-generated.json", "w+") as fobj:
                 fobj.write(json.dumps(input_params, indent=4))
@@ -1222,6 +1418,8 @@ def main():
     # args = parser.parse_args()
     html_report_name = "CVIAST-report.html"
     quiet_mode = True
+    is_timeoutpresent = False
+    properties_file_path = None
     if sys.argv[1:]:
         for _ in sys.argv[1:]:
             if "report-out" in _:
@@ -1230,6 +1428,21 @@ def main():
             if "--v" in _:
                 quiet_mode = False
                 sys.argv.remove(_)
+            if "timeout" in _:
+                is_timeoutpresent = True
+            if "--properties=" in _:
+                properties_file_path = str(_).split("=")[1]
+        if not properties_file_path:
+            try:
+                prop_file_index = sys.argv.index("properties")
+                properties_file_path = sys.argv[prop_file_index]
+            except ValueError:
+                print("DEBUG: properties file not passed!")
+        if properties_file_path:
+            if os.path.exists(properties_file_path):
+                shutil.copy(properties_file_path,"properties.yaml")
+        if not is_timeoutpresent:
+            sys.argv.append("--timeout=300")
         if sys.argv[1:][0] == "--list-supported-fuzz-types":
             print("\n\t\t\t\t\t\t".join(supoorted_fuzz_types))
             return
@@ -1328,7 +1541,7 @@ def main():
     for _ in os.listdir("tests"):
         if ".py" in _:
             os.system("autopep8 --in-place --aggressive --aggressive tests/"+str(_))
-            printProgressBar(i + 1, len(os.listdir("tests")), prefix='Progress:', suffix='Complete', length=50)
+            printProgressBar(i + 1, len([_ for _ in os.listdir("tests") if ".py" in _]), prefix='Progress:', suffix='Complete', length=50)
             i+=1
     print("\n\n\t\t..........done")
 
